@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_alpha_instr.c,v 1.11 2006/06/03 06:46:44 debug Exp $
+ *  $Id: cpu_alpha_instr.c,v 1.13 2006/07/26 23:21:47 debug Exp $
  *
  *  Alpha instructions.
  *
@@ -65,7 +65,6 @@ X(call_pal)
 	alpha_palcode(cpu, ic->arg[0]);
 
 	if (!cpu->running) {
-		cpu->running_translated = 0;
 		cpu->n_translated_instrs --;
 		cpu->cd.alpha.next_ic = &nothing_call;
 	} else if (cpu->pc != old_pc) {
@@ -879,8 +878,7 @@ X(to_be_translated)
 		}
 		ic->f = alpha_loadstore[
 		    loadstore_type + (imm==0? 4 : 0) + 8 * load
-		    + (cpu->machine->dyntrans_alignment_check? 16:0)
-		    + 32 * llsc];
+		    + 16 * llsc];
 		/*  Load to the zero register is treated as a prefetch
 		    hint. It is ignored here.  */
 		if (load && ra == ALPHA_ZERO) {
