@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_mips.c,v 1.58 2006/06/24 21:47:23 debug Exp $
+ *  $Id: cpu_mips.c,v 1.63 2006/08/12 11:43:13 debug Exp $
  *
  *  MIPS core CPU emulation.
  */
@@ -35,6 +35,7 @@
 #include <string.h>
 #include <sys/types.h>
 #include <ctype.h>
+#include <unistd.h>
 
 #include "../../config.h"
 
@@ -144,12 +145,14 @@ int mips_cpu_new(struct cpu *cpu, struct memory *mem, struct machine *machine,
 		cpu->is_32bit = 1;
 
 	if (cpu->is_32bit) {
+		cpu->run_instr = mips32_run_instr;
 		cpu->update_translation_table = mips32_update_translation_table;
 		cpu->invalidate_translation_caches =
 		    mips32_invalidate_translation_caches;
 		cpu->invalidate_code_translation =
 		    mips32_invalidate_code_translation;
 	} else {
+		cpu->run_instr = mips_run_instr;
 		cpu->update_translation_table = mips_update_translation_table;
 		cpu->invalidate_translation_caches =
 		    mips_invalidate_translation_caches;
@@ -747,6 +750,10 @@ void mips_cpu_register_match(struct machine *m, char *name,
 	}
 
 	/*  TODO: Coprocessor 1,2,3 registers.  */
+
+	/*  Only return lowest 32 bits when doing 32-bit emulation:  */
+	if (!writeflag && m->cpus[cpunr]->is_32bit)
+		*valuep = (uint32_t) (*valuep);
 }
 
 
@@ -1597,7 +1604,6 @@ void mips_cpu_register_dump(struct cpu *cpu, int gprs, int coprocs)
 		}
 
 		/*  Coprocessor registers:  */
-		/*  TODO: multiple selections per register?  */
 		for (i=0; i<32; i++) {
 			/*  32-bit:  */
 			if ((i & nm1) == 0)
@@ -1873,6 +1879,16 @@ void mips_cpu_exception(struct cpu *cpu, int exccode, int tlb, uint64_t vaddr,
 	uint64_t *reg = &cpu->cd.mips.coproc[0]->reg[0];
 	int exc_model = cpu->cd.mips.cpu_type.exc_model;
 
+	if (cpu->is_halted) {
+		/*
+		 *  If the exception occurred on a 'wait' instruction, then let
+		 *  the instruction following the wait instruction be the one
+		 *  we continue at when the interrupt service routine returns.
+		 */
+		cpu->is_halted = 0;
+		cpu->pc += sizeof(uint32_t);
+	}
+
 	if (!quiet_mode) {
 		uint64_t offset;
 		int x;
