@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: debugger.c,v 1.12 2006/06/25 01:25:32 debug Exp $
+ *  $Id: debugger.c,v 1.14 2006/07/01 21:15:46 debug Exp $
  *
  *  Single-step debugger.
  *
@@ -89,7 +89,6 @@ extern int quiet_mode;
 volatile int single_step = NOT_SINGLE_STEPPING;
 volatile int exit_debugger;
 int force_debugger_at_exit = 0;
-int show_opcode_statistics = 0;
 
 volatile int single_step_breakpoint = 0;
 int debugger_n_steps_left_before_interaction = 0;
@@ -860,7 +859,7 @@ void debugger(void)
 	}
 
 	gettimeofday(&debugger_machine->starttime, NULL);
-	debugger_machine->ncycles_since_gettimeofday = 0;
+	debugger_machine->ninstrs_since_gettimeofday = 0;
 
 	single_step = NOT_SINGLE_STEPPING;
 	debugger_machine->instruction_trace = old_instruction_trace;
