@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_dec21143.c,v 1.24 2006/02/09 20:02:59 debug Exp $
+ *  $Id: dev_dec21143.c,v 1.25 2006/03/04 12:38:47 debug Exp $
  *
  *  DEC 21143 ("Tulip") ethernet controller. Implemented from Intel document
  *  278074-001 ("21143 PC/CardBus 10/100Mb/s Ethernet LAN Controller") and by
@@ -1012,7 +1012,7 @@ DEVINIT(dec21143)
 	    devinit->addr, 0x100, dev_dec21143_access, d, DM_DEFAULT, NULL);
 
 	machine_add_tickfunction(devinit->machine,
-	    dev_dec21143_tick, d, DEC21143_TICK_SHIFT);
+	    dev_dec21143_tick, d, DEC21143_TICK_SHIFT, 0.0);
 
 	/*
 	 *  NetBSD/cats uses memory accesses, OpenBSD/cats uses I/O registers.
