@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_fb.c,v 1.118 2006/03/04 12:38:47 debug Exp $
+ *  $Id: dev_fb.c,v 1.123 2006/07/24 08:08:39 debug Exp $
  *  
  *  Generic framebuffer device.
  *
@@ -34,7 +34,8 @@
  *	DECstation Maxine, 1024x768 8-bit color
  *	HPC (mips, arm, ..) framebuffer
  *	Playstation 2 (24-bit color)
- *	generic (any resolution, several bit depths possible)
+ *	Generic (any resolution, several bit depths possible, useful for
+ *		testmachines)
  *
  *
  *  TODO:  This should actually be independent of X11, but that
@@ -62,7 +63,7 @@
 #endif
 
 
-#define	FB_TICK_SHIFT		18
+#define	FB_TICK_SHIFT		19
 
 
 /*  #define FB_DEBUG  */
@@ -104,11 +105,23 @@ void set_blackwhite_palette(struct vfb_data *d, int ncolors)
 }
 
 
+static void set_title(struct vfb_data *d)
+{
+	snprintf(d->title, sizeof(d->title),"GXemul: %ix%ix%i %s framebuffer",
+	    d->visible_xsize, d->visible_ysize, d->bit_depth, d->name);
+	d->title[sizeof(d->title)-1] = '\0';
+}
+
+
 /*
  *  dev_fb_resize():
  *
  *  Resize a framebuffer window. (This functionality is probably a bit buggy,
  *  because I didn't think of including it from the start.)
+ *
+ *  SUPER-IMPORTANT: Anyone who resizes a framebuffer by calling this function
+ *  must also clear all dyntrans address translations manually, in all cpus
+ *  which might have access to the framebuffer!
  */
 void dev_fb_resize(struct vfb_data *d, int new_xsize, int new_ysize)
 {
@@ -121,6 +134,11 @@ void dev_fb_resize(struct vfb_data *d, int new_xsize, int new_ysize)
 		return;
 	}
 
+	if (new_xsize < 10 || new_ysize < 10) {
+		fatal("dev_fb_resize(): size too small.\n");
+		exit(1);
+	}
+
 	new_bytes_per_line = new_xsize * d->bit_depth / 8;
 	size = new_ysize * new_bytes_per_line;
 
@@ -163,9 +181,15 @@ void dev_fb_resize(struct vfb_data *d, int new_xsize, int new_ysize)
 	d->x11_xsize = d->xsize / d->vfb_scaledown;
 	d->x11_ysize = d->ysize / d->vfb_scaledown;
 
+	memory_device_update_data(d->memory, d, d->framebuffer);
+
+	set_title(d);
+
 #ifdef WITH_X11
-	if (d->fb_window != NULL)
+	if (d->fb_window != NULL) {
 		x11_fb_resize(d->fb_window, new_xsize, new_ysize);
+		x11_set_standard_properties(d->fb_window, d->title);
+	}
 #endif
 }
 
@@ -211,7 +235,7 @@ void dev_fb_setcursor(struct vfb_data *d, int cursor_x, int cursor_y, int on,
  *  block copy/fill.
  *
  *  If fillflag is non-zero, then fill_[rgb] should contain the color
- *  with which to fill.
+ *  with which to fill. (In 8-bit mode, only fill_r is used.)
  *
  *  If fillflag is zero, copy mode is used, and from_[xy] should contain
  *  the offset on the framebuffer where we should copy from.
@@ -222,7 +246,7 @@ void framebuffer_blockcopyfill(struct vfb_data *d, int fillflag, int fill_r,
 	int fill_g, int fill_b, int x1, int y1, int x2, int y2,
 	int from_x, int from_y)
 {
-	int y;
+	int x, y;
 	long from_ofs, dest_ofs, linelen;
 
 	if (fillflag)
@@ -245,22 +269,23 @@ void framebuffer_blockcopyfill(struct vfb_data *d, int fillflag, int fill_r,
 	if (fillflag) {
 		for (y=y1; y<=y2; y++) {
 			if (y>=0 && y<d->ysize) {
-				int x;
-				char buf[8192 * 3];
-				if (d->bit_depth == 24)
+				unsigned char *buf =
+				    d->framebuffer + dest_ofs;
+
+				if (d->bit_depth == 24) {
 					for (x=0; x<linelen && x<sizeof(buf);
 					    x += 3) {
 						buf[x] = fill_r;
 						buf[x+1] = fill_g;
 						buf[x+2] = fill_b;
 					}
-				else {
-					fatal("[ fb: TODO: fill for non-24-bit"
-					    " modes ]\n");
+				} else if (d->bit_depth == 8) {
+					memset(buf, fill_r, linelen);
+				} else {
+					fatal("Unimplemented bit-depth (%i)"
+					    " for fb fill\n", d->bit_depth);
+					exit(1);
 				}
-
-				memmove(d->framebuffer + dest_ofs, buf,
-				    linelen);
 			}
 
 			dest_ofs += d->bytes_per_line;
@@ -268,12 +293,16 @@ void framebuffer_blockcopyfill(struct vfb_data *d, int fillflag, int fill_r,
 	} else {
 		from_ofs = d->bytes_per_line * from_y +
 		    (d->bit_depth/8) * from_x;
-
 		for (y=y1; y<=y2; y++) {
-			if (y>=0 && y<d->ysize)
-				memmove(d->framebuffer + dest_ofs,
-				    d->framebuffer + from_ofs, linelen);
-
+			if (y >= 0 && y < d->ysize) {
+				if (from_y >= 0 && from_y < d->ysize)
+					memmove(d->framebuffer + dest_ofs,
+					    d->framebuffer + from_ofs, linelen);
+				else
+					memset(d->framebuffer + dest_ofs,
+					    0, linelen);
+			}
+			from_y ++;
 			from_ofs += d->bytes_per_line;
 			dest_ofs += d->bytes_per_line;
 		}
@@ -728,7 +757,6 @@ struct vfb_data *dev_fb_init(struct machine *machine, struct memory *mem,
 	size_t size, nlen;
 	int flags;
 	int reverse_start = 0;
-	char title[400];
 	char *name2;
 
 	d = malloc(sizeof(struct vfb_data));
@@ -743,6 +771,7 @@ struct vfb_data *dev_fb_init(struct machine *machine, struct memory *mem,
 		reverse_start = 1;
 	}
 
+	d->memory = mem;
 	d->vfb_type = vfb_type;
 
 	/*  Defaults:  */
@@ -821,20 +850,14 @@ struct vfb_data *dev_fb_init(struct machine *machine, struct memory *mem,
 		d->update_x2 = d->update_y2 = -1;
 	}
 
-	/*  Don't set the title to include the size of the framebuffer for
-	    VGA, since then the resolution might change during runtime.  */
-	if (strcmp(name, "VGA") == 0)
-		snprintf(title, sizeof(title),"GXemul: %s framebuffer", name);
-	else
-		snprintf(title, sizeof(title),"GXemul: %ix%ix%i %s framebuffer",
-		    d->visible_xsize, d->visible_ysize, d->bit_depth, name);
-	title[sizeof(title)-1] = '\0';
+	d->name = strdup(name);
+	set_title(d);
 
 #ifdef WITH_X11
 	if (machine->use_x11) {
 		int i = 0;
 		d->fb_window = x11_fb_init(d->x11_xsize, d->x11_ysize,
-		    title, machine->x11_scaledown, machine);
+		    d->title, machine->x11_scaledown, machine);
 		switch (d->fb_window->x11_screen_depth) {
 		case 15: i = 2; break;
 		case 16: i = 4; break;
