@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_pcic.c,v 1.14 2005/11/13 00:14:09 debug Exp $
+ *  $Id: dev_pcic.c,v 1.16 2006/02/09 20:02:59 debug Exp $
  *
  *  Intel 82365SL PC Card Interface Controller (called "pcic" by NetBSD).
  *
@@ -60,9 +60,7 @@ struct pcic_data {
 /*
  *  dev_pcic_cis_access():
  */
-int dev_pcic_cis_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(pcic_cis)
 {
 	/*  struct pcic_data *d = (struct pcic_data *) extra;  */
 	uint64_t idata = 0, odata = 0;
@@ -145,9 +143,7 @@ int dev_pcic_cis_access(struct cpu *cpu, struct memory *mem,
 /*
  *  dev_pcic_access():
  */
-int dev_pcic_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(pcic)
 {
 	struct pcic_data *d = (struct pcic_data *) extra;
 	uint64_t idata = 0, odata = 0;
@@ -214,10 +210,7 @@ int dev_pcic_access(struct cpu *cpu, struct memory *mem,
 }
 
 
-/*
- *  devinit_pcic():
- */
-int devinit_pcic(struct devinit *devinit)
+DEVINIT(pcic)
 {
 	struct pcic_data *d = malloc(sizeof(struct pcic_data));
 
