@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2005-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_ps2_ether.c,v 1.4 2005/11/13 00:14:09 debug Exp $
+ *  $Id: dev_ps2_ether.c,v 1.6 2006/02/09 20:02:59 debug Exp $
  *  
  *  Playstation 2 ethernet (smap and emac3).
  *
@@ -53,9 +53,7 @@ struct ps2_ether_data {
 /*
  *  dev_ps2_ether_access():
  */
-int dev_ps2_ether_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(ps2_ether)
 {
 	/*  struct ps2_ether_data *d = extra;  */
 	uint64_t idata = 0, odata = 0;
@@ -87,10 +85,7 @@ int dev_ps2_ether_access(struct cpu *cpu, struct memory *mem,
 }
 
 
-/*
- *  devinit_ps2_ether():
- */
-int devinit_ps2_ether(struct devinit *devinit)
+DEVINIT(ps2_ether)
 {
 	struct ps2_ether_data *d;
 
