@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2004-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_sn.c,v 1.13 2005/11/13 00:14:10 debug Exp $
+ *  $Id: dev_sn.c,v 1.15 2006/02/09 20:02:59 debug Exp $
  *  
  *  National Semiconductor SONIC ("sn") DP83932 ethernet.
  *
@@ -60,9 +60,7 @@ struct sn_data {
 /*
  *  dev_sn_access():
  */
-int dev_sn_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(sn)
 {
 	struct sn_data *d = (struct sn_data *) extra;
 	uint64_t idata = 0, odata = 0;
@@ -99,10 +97,7 @@ int dev_sn_access(struct cpu *cpu, struct memory *mem,
 }
 
 
-/*
- *  devinit_sn():
- */
-int devinit_sn(struct devinit *devinit)
+DEVINIT(sn)
 {
 	char *name2;
 	size_t nlen = 55;
