@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2005  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2006  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: dev_turbochannel.c,v 1.45 2005/11/13 00:14:10 debug Exp $
+ *  $Id: dev_turbochannel.c,v 1.46 2006/01/01 13:17:17 debug Exp $
  *  
  *  Generic framework for TURBOchannel devices, used in DECstation machines.
  */
@@ -66,9 +66,7 @@ struct turbochannel_data {
 /*
  *  dev_turbochannel_access():
  */
-int dev_turbochannel_access(struct cpu *cpu, struct memory *mem,
-	uint64_t relative_addr, unsigned char *data, size_t len,
-	int writeflag, void *extra)
+DEVICE_ACCESS(turbochannel)
 {
 	struct turbochannel_data *d = extra;
 	uint64_t idata = 0, odata = 0;
