@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: emul.c,v 1.255 2006/06/24 19:52:27 debug Exp $
+ *  $Id: emul.c,v 1.260 2006/07/26 23:21:47 debug Exp $
  *
  *  Emulation startup and misc. routines.
  */
@@ -1271,7 +1271,7 @@ void emul_machine_setup(struct machine *m, int n_load, char **load_names,
 			break;
 
 		case ARCH_SH:
-			if (cpu->cd.sh.bits == 32)
+			if (cpu->cd.sh.cpu_type.bits == 32)
 				cpu->pc &= 0xffffffffULL;
 			cpu->pc &= ~1;
 			break;
@@ -1279,6 +1279,10 @@ void emul_machine_setup(struct machine *m, int n_load, char **load_names,
 		case ARCH_SPARC:
 			break;
 
+		case ARCH_TRANSPUTER:
+			cpu->pc &= 0xffffffffULL;
+			break;
+
 		case ARCH_X86:
 			/*
 			 *  NOTE: The toc field is used to indicate an ELF32
@@ -1332,8 +1336,7 @@ void emul_machine_setup(struct machine *m, int n_load, char **load_names,
 		useremul_setup(cpu, n_load, load_names);
 
 	/*  Startup the bootstrap CPU:  */
-	cpu->bootstrap_cpu_flag = 1;
-	cpu->running            = 1;
+	cpu->running = 1;
 
 	/*  ... or pause all CPUs, if start_paused is set:  */
 	if (m->start_paused) {
@@ -1582,22 +1585,22 @@ void emul_run(struct emul **emuls, int n_emuls)
 		go = 0;
 
 		/*  Flush X11 and serial console output every now and then:  */
-		if (emuls[0]->machines[0]->ncycles >
-		    emuls[0]->machines[0]->ncycles_flush + (1<<18)) {
+		if (emuls[0]->machines[0]->ninstrs >
+		    emuls[0]->machines[0]->ninstrs_flush + (1<<19)) {
 			x11_check_event(emuls, n_emuls);
 			console_flush();
-			emuls[0]->machines[0]->ncycles_flush =
-			    emuls[0]->machines[0]->ncycles;
+			emuls[0]->machines[0]->ninstrs_flush =
+			    emuls[0]->machines[0]->ninstrs;
 		}
 
-		if (emuls[0]->machines[0]->ncycles >
-		    emuls[0]->machines[0]->ncycles_show + (1<<25)) {
-			emuls[0]->machines[0]->ncycles_since_gettimeofday +=
-			    (emuls[0]->machines[0]->ncycles -
-			     emuls[0]->machines[0]->ncycles_show);
+		if (emuls[0]->machines[0]->ninstrs >
+		    emuls[0]->machines[0]->ninstrs_show + (1<<25)) {
+			emuls[0]->machines[0]->ninstrs_since_gettimeofday +=
+			    (emuls[0]->machines[0]->ninstrs -
+			     emuls[0]->machines[0]->ninstrs_show);
 			cpu_show_cycles(emuls[0]->machines[0], 0);
-			emuls[0]->machines[0]->ncycles_show =
-			    emuls[0]->machines[0]->ncycles;
+			emuls[0]->machines[0]->ninstrs_show =
+			    emuls[0]->machines[0]->ninstrs;
 		}
 
 		if (single_step == ENTER_SINGLE_STEPPING) {
