@@ -28,7 +28,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: cpu_mips.h,v 1.43 2006/06/24 21:47:24 debug Exp $
+ *  $Id: cpu_mips.h,v 1.45 2006/07/16 13:32:27 debug Exp $
  */
 
 #include "misc.h"
@@ -69,6 +69,9 @@ struct mips_cpu_type_def {
 
 /*
  *  Coproc 0:
+ *
+ *  NOTE:
+ *	On R3000, only hi and lo0 are used, and then only the lowest 32 bits.
  */
 #define	N_MIPS_COPROC_REGS	32
 struct mips_tlb {
@@ -365,10 +368,12 @@ void mips_unaligned_loadstore(struct cpu *cpu, struct mips_instr_call *ic,
 	int is_left, int wlen, int store);
 
 
+int mips_run_instr(struct cpu *cpu);
 void mips_update_translation_table(struct cpu *cpu, uint64_t vaddr_page,
 	unsigned char *host_page, int writeflag, uint64_t paddr_page);
 void mips_invalidate_translation_caches(struct cpu *cpu, uint64_t, int);
 void mips_invalidate_code_translation(struct cpu *cpu, uint64_t, int);
+int mips32_run_instr(struct cpu *cpu);
 void mips32_update_translation_table(struct cpu *cpu, uint64_t vaddr_page,
 	unsigned char *host_page, int writeflag, uint64_t paddr_page);
 void mips32_invalidate_translation_caches(struct cpu *cpu, uint64_t, int);
