@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: machine_cobalt.c,v 1.3 2006/06/24 10:19:19 debug Exp $
+ *  $Id: machine_cobalt.c,v 1.4 2006/08/12 19:31:36 debug Exp $
  */
 
 #include <stdio.h>
@@ -136,6 +136,12 @@ MACHINE_DEFAULT_CPU(cobalt)
 }
 
 
+MACHINE_DEFAULT_RAM(cobalt)
+{
+	machine->physical_ram_in_mb = 64;
+}
+
+
 MACHINE_REGISTER(cobalt)
 {
 	MR_DEFAULT(cobalt, "Cobalt", ARCH_MIPS, MACHINE_COBALT);
