@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *   
  *
- *  $Id: machine_test.c,v 1.14 2006/06/24 10:19:19 debug Exp $
+ *  $Id: machine_test.c,v 1.18 2006/07/25 19:35:28 debug Exp $
  *
  *  Various "test" machines (bare machines with just a CPU, or a bare machine
  *  plus some experimental devices).
@@ -60,8 +60,9 @@ static void default_test(struct machine *machine, struct cpu *cpu)
 	    (uint64_t) DEV_MP_ADDRESS);
 	device_add(machine, tmpstr);
 
-	dev_fb_init(machine, machine->memory, DEV_FB_ADDRESS, VFB_GENERIC,
-	    640,480, 640,480, 24, "generic"); 
+	snprintf(tmpstr, sizeof(tmpstr), "fbctrl addr=0x%"PRIx64,
+	    (uint64_t) DEV_FBCTRL_ADDRESS);
+	device_add(machine, tmpstr);
 
 	snprintf(tmpstr, sizeof(tmpstr), "disk addr=0x%"PRIx64,
 	    (uint64_t) DEV_DISK_ADDRESS);
@@ -409,8 +410,9 @@ MACHINE_SETUP(testmips)
 	    (uint64_t) DEV_MP_ADDRESS);
 	device_add(machine, tmpstr);
 
-	dev_fb_init(machine, machine->memory, DEV_FB_ADDRESS, VFB_GENERIC,
-	    640,480, 640,480, 24, "testmips generic"); 
+	snprintf(tmpstr, sizeof(tmpstr), "fbctrl addr=0x%"PRIx64,
+	    (uint64_t) DEV_FBCTRL_ADDRESS);
+	device_add(machine, tmpstr);
 
 	snprintf(tmpstr, sizeof(tmpstr), "disk addr=0x%"PRIx64,
 	    (uint64_t) DEV_DISK_ADDRESS);
@@ -519,13 +521,13 @@ MACHINE_SETUP(testsh)
 
 MACHINE_DEFAULT_CPU(baresh)
 {
-	machine->cpu_name = strdup("SH");
+	machine->cpu_name = strdup("SH5");
 }
 
 
 MACHINE_DEFAULT_CPU(testsh)
 {
-	machine->cpu_name = strdup("SH");
+	machine->cpu_name = strdup("SH5");
 }
 
 
@@ -593,3 +595,25 @@ MACHINE_REGISTER(testsparc)
 	machine_entry_add_alias(me, "testsparc");
 }
 
+
+MACHINE_SETUP(baretransputer)
+{
+	machine->machine_name = "Generic \"bare\" Transputer machine";
+	machine->stable = 1;
+}
+
+
+MACHINE_DEFAULT_CPU(baretransputer)
+{
+	machine->cpu_name = strdup("T800");
+}
+
+
+MACHINE_REGISTER(baretransputer)
+{
+	MR_DEFAULT(baretransputer, "Generic \"bare\" Transputer machine",
+	    ARCH_TRANSPUTER, MACHINE_BARETRANSPUTER);
+
+	machine_entry_add_alias(me, "baretransputer");
+}
+
