@@ -25,7 +25,7 @@
  *  SUCH DAMAGE.
  *
  *
- *  $Id: arcbios.c,v 1.8 2006/04/15 08:21:07 debug Exp $
+ *  $Id: arcbios.c,v 1.10 2006/07/26 23:21:48 debug Exp $
  *
  *  ARCBIOS emulation.
  */
@@ -1123,7 +1123,6 @@ void arcbios_private_emul(struct cpu *cpu)
 		debug("\n");
 		fatal("ARCBIOS: unimplemented PRIVATE vector 0x%x\n", vector);
 		cpu->running = 0;
-		cpu->dead = 1;
 	}
 }
 
@@ -1190,7 +1189,6 @@ int arcbios_emul(struct cpu *cpu)
 		/*  Halt all CPUs.  */
 		for (i=0; i<machine->ncpus; i++) {
 			machine->cpus[i]->running = 0;
-			machine->cpus[i]->dead = 1;
 		}
 		machine->exit_without_entering_debugger = 1;
 		break;
@@ -1854,7 +1852,6 @@ int arcbios_emul(struct cpu *cpu)
 		quiet_mode = 0;
 		cpu_register_dump(machine, cpu, 1, 0x1);
 		cpu->running = 0;
-		cpu->dead = 1;
 		break;
 	default:
 		quiet_mode = 0;
@@ -1864,7 +1861,6 @@ int arcbios_emul(struct cpu *cpu)
 		debug("\n");
 		fatal("ARCBIOS: unimplemented vector 0x%x\n", vector);
 		cpu->running = 0;
-		cpu->dead = 1;
 	}
 
 	return 1;
@@ -2267,10 +2263,10 @@ static void arc_environment_setup(struct machine *machine, int is64bit,
 		if (machine->machine_type == MACHINE_SGI) {
 			if (machine->machine_subtype == 30)
 				strlcat(init_bootpath, "xio(0)pci(15)",
-				    MACHINE_NAME_MAXBUF);
+				    bootpath_len);
 			if (machine->machine_subtype == 32)
 				strlcat(init_bootpath, "pci(0)",
-				    MACHINE_NAME_MAXBUF);
+				    bootpath_len);
 		}
 
 		if (diskimage_is_a_cdrom(machine, machine->bootdev_id,
@@ -2286,7 +2282,7 @@ static void arc_environment_setup(struct machine *machine, int is64bit,
 	}
 
 	if (machine->machine_type == MACHINE_ARC)
-		strlcat(init_bootpath, "\\", MACHINE_NAME_MAXBUF);
+		strlcat(init_bootpath, "\\", bootpath_len);
 
 	machine->bootstr = malloc(ARC_BOOTSTR_BUFLEN);
 		if (machine->bootstr == NULL) {
